from telicent_labels.security_labels import Label, MultiValueLabel, SecurityLabelBuilder, SingleValueLabel
from telicent_labels.telicent_model import TelicentModel
from telicent_labels.telicentv1 import TelicentLabelsV1
from telicent_labels.telicentv2 import TelicentSecurityLabelsV2

__license__ = """
Copyright (c) Telicent Ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


__all__ = [
    "SecurityLabelBuilder",
    "TelicentSecurityLabelsV2",
    "TelicentLabelsV1",
    "MultiValueLabel",
    "SingleValueLabel",
    "Label",
    "TelicentModel",
]
